@@ -3,118 +3,47 @@ package CHI::Driver::DBI;
 use strict;
 use warnings;
 
-use DBI;
 use DBI::Const::GetInfoType;
 use Moose;
+use Moose::Util::TypeConstraints;
 use Carp qw(croak);
 
-our $VERSION = '1.21';
-
-# TODO:  For pg see "upsert" - http://www.postgresql.org/docs/current/static/plpgsql-control-structures.html#PLPGSQL-UPSERT-EXAMPLE
+our $VERSION = '1.24';
 
 extends 'CHI::Driver';
 
-=head1 NAME
+my $type = "CHI::Driver::DBI";
 
-CHI::Driver::DBI - Use DBI for cache storage
+subtype "$type.DBIHandleGenerator" => as 'CodeRef';
+subtype "$type.DBIXConnector"      => as 'DBIx::Connector';
+subtype "$type.DBIHandle"          => as 'DBI::db';
 
-=head1 SYNOPSIS
+coerce "$type.DBIHandleGenerator" => from "$type.DBIXConnector" => via {
+    my $dbixconn = $_;
+    sub { $dbixconn->dbh }
+};
+coerce "$type.DBIHandleGenerator" => from "$type.DBIHandle" => via {
+    my $dbh = $_;
+    sub { $dbh }
+};
 
- use CHI;
-
- my $dbh   = DBI->connect(...);
- my $cache = CHI->new( driver => 'DBI', dbh => $dbh, );
- OR
- my $cache = CHI->new( driver => 'DBI', dbh => $dbh, dbh_ro => $dbh_ro, );
-
-=head1 DESCRIPTION
-
-This driver uses a database table to store the cache.  The newest versions of
-MySQL and SQLite work are known to work.  Other RDBMSes should work.
-
-Why cache things in a database?  Isn't the database what people are trying to
-avoid with caches?  This is often true, but a simple primary key lookup is
-extremely fast in many databases and this provides a shared cache that can be
-used when less reliable storage like memcached is not appropriate.  Also, the
-speed of simple lookups on MySQL when accessed over a local socket is very hard
-to beat.  DBI is fast.
-
-=head1 ATTRIBUTES
-
-=over
-
-=item namespace
-
-The namespace you pass in will be appended to the C<table_prefix> and used as a
-table name.  That means that if you don't specify a namespace or table_prefix
-the cache will be stored in a table called C<chi_Default>.
-
-=item table_prefix
-
-This is the prefix that is used when building a table name.  If you want to
-just use the namespace as a literal table name, set this to undef.  Defaults to
-C<chi_>.
-
-=cut
-
-has 'table_prefix' => ( is => 'rw', isa => 'Str', default => 'chi_', );
-
-=item dbh
-
-The main, or rw, DBI handle used to communicate with the db. If a dbh_ro handle
-is defined then this handle will only be used for writing.
-
-This attribute can be set after object creation as well, so in a persistent
-environment like mod_perl or FastCGI you may keep an instance of the cache
-around and set the dbh on each request after checking it with ping().
-
-=cut
-
-has 'dbh' => ( is => 'rw', isa => 'DBI::db', required => 1, );
-
-=item dbh_ro
-
-The optional DBI handle used for read-only operations.  This is to support
-master/slave RDBMS setups.
-
-=cut
-
-has 'dbh_ro' => ( is => 'rw', isa => 'DBI::db', );
-
-=item sql_strings
-
-Hashref of SQL strings to use in the different cache operations. The strings
-are built depending on the RDBMS that dbh is attached to.
-
-=back
-
-=cut
-
-has 'sql_strings' => ( is => 'rw', isa => 'HashRef', lazy_build => 1, );
+has 'dbh'          => ( is => 'ro', isa => "$type.DBIHandleGenerator", coerce => 1 );
+has 'dbh_ro'       => ( is => 'ro', isa => "$type.DBIHandleGenerator", predicate => 'has_dbh_ro', coerce => 1 );
+has 'sql_strings'  => ( is => 'rw', isa => 'HashRef', lazy_build => 1 );
+has 'table_prefix' => ( is => 'rw', isa => 'Str', default => 'chi_' );
 
 __PACKAGE__->meta->make_immutable;
 
-=head1 METHODS
-
-=over
-
-=item BUILD
-
-Standard issue Moose BUILD method.  Used to build the sql_strings.  If the
-parameter C<create_table> to C<new()> was set to true, it will attempt to
-create the db table.  For Mysql and SQLite the statement is "create if not
-exists..." so it's generally harmless.
-
-=cut
-
 sub BUILD {
-    my ( $self, $args, ) = @_;
+    my ( $self, $args ) = @_;
+
+    my $dbh = $self->dbh->();
 
     $self->sql_strings;
 
     if ( $args->{create_table} ) {
-        $self->{dbh}->do( $self->{sql_strings}->{create} )
-          or croak $self->{dbh}->errstr;
+        $dbh->do( $self->sql_strings->{create} )
+          or croak $dbh->errstr;
     }
 
     return;
@@ -129,10 +58,11 @@ sub _table {
 sub _build_sql_strings {
     my ( $self, ) = @_;
 
-    my $table   = $self->dbh->quote_identifier( $self->_table );
-    my $value   = $self->dbh->quote_identifier('value');
-    my $key     = $self->dbh->quote_identifier('key');
-    my $db_name = $self->dbh->get_info( $GetInfoType{SQL_DBMS_NAME} );
+    my $dbh     = $self->dbh->();
+    my $table   = $dbh->quote_identifier( $self->_table );
+    my $value   = $dbh->quote_identifier('value');
+    my $key     = $dbh->quote_identifier('key');
+    my $db_name = $dbh->get_info( $GetInfoType{SQL_DBMS_NAME} );
 
     my $strings = {
         fetch    => "SELECT $value FROM $table WHERE $key = ?",
@@ -142,7 +72,7 @@ sub _build_sql_strings {
         clear    => "DELETE FROM $table",
         get_keys => "SELECT DISTINCT $key FROM $table",
         create   => "CREATE TABLE IF NOT EXISTS $table ("
-          . " $key VARCHAR( 600 ), $value TEXT,"
+          . " $key VARCHAR( 300 ), $value TEXT,"
           . " PRIMARY KEY ( $key ) )",
     };
 
@@ -165,15 +95,11 @@ sub _build_sql_strings {
     return $strings;
 }
 
-=item fetch
-
-=cut
-
 sub fetch {
     my ( $self, $key, ) = @_;
 
-    my $dbh = $self->{dbh_ro} ? $self->{dbh_ro} : $self->{dbh};
-    my $sth = $dbh->prepare_cached( $self->{sql_strings}->{fetch} )
+    my $dbh = $self->has_dbh_ro ? $self->dbh_ro->() : $self->dbh->();
+    my $sth = $dbh->prepare_cached( $self->sql_strings->{fetch} )
       or croak $dbh->errstr;
     $sth->execute($key) or croak $sth->errstr;
     my $results = $sth->fetchall_arrayref;
@@ -181,19 +107,15 @@ sub fetch {
     return $results->[0]->[0];
 }
 
-=item store
-
-=cut
-
 sub store {
     my ( $self, $key, $data, ) = @_;
 
-    my $sth = $self->{dbh}->prepare_cached( $self->{sql_strings}->{store} );
+    my $dbh = $self->dbh->();
+    my $sth = $dbh->prepare_cached( $self->sql_strings->{store} );
     if ( not $sth->execute( $key, $data ) ) {
-        if ( $self->{sql_strings}->{store2} ) {
-            my $sth =
-              $self->{dbh}->prepare_cached( $self->{sql_strings}->{store2} )
-              or croak $self->{dbh}->errstr;
+        if ( $self->sql_strings->{store2} ) {
+            my $sth = $dbh->prepare_cached( $self->sql_strings->{store2} )
+              or croak $dbh->errstr;
             $sth->execute( $data, $key )
               or croak $sth->errstr;
         }
@@ -206,45 +128,35 @@ sub store {
     return;
 }
 
-=item remove
-
-=cut
-
 sub remove {
     my ( $self, $key, ) = @_;
 
-    my $sth = $self->dbh->prepare_cached( $self->{sql_strings}->{remove} )
-      or croak $self->{dbh}->errstr;
+    my $dbh = $self->dbh->();
+    my $sth = $dbh->prepare_cached( $self->sql_strings->{remove} )
+      or croak $dbh->errstr;
     $sth->execute($key) or croak $sth->errstr;
     $sth->finish;
 
     return;
 }
 
-=item clear
-
-=cut
-
 sub clear {
     my ( $self, $key, ) = @_;
 
-    my $sth = $self->{dbh}->prepare_cached( $self->{sql_strings}->{clear} )
-      or croak $self->{dbh}->errstr;
+    my $dbh = $self->dbh->();
+    my $sth = $dbh->prepare_cached( $self->sql_strings->{clear} )
+      or croak $dbh->errstr;
     $sth->execute() or croak $sth->errstr;
     $sth->finish();
 
     return;
 }
 
-=item get_keys
-
-=cut
-
 sub get_keys {
     my ( $self, ) = @_;
 
-    my $dbh = $self->{dbh_ro} ? $self->{dbh_ro} : $self->{dbh};
-    my $sth = $dbh->prepare_cached( $self->{sql_strings}->{get_keys} )
+    my $dbh = $self->has_dbh_ro ? $self->dbh_ro->() : $self->dbh->();
+    my $sth = $dbh->prepare_cached( $self->sql_strings->{get_keys} )
       or croak $dbh->errstr;
     $sth->execute() or croak $sth->errstr;
     my $results = $sth->fetchall_arrayref( [0] );
@@ -253,17 +165,125 @@ sub get_keys {
     return @{$results};
 }
 
-=item get_namespaces
+sub get_namespaces { croak 'not supported' }
 
-Not supported at this time.
+# TODO:  For pg see "upsert" - http://www.postgresql.org/docs/current/static/plpgsql-control-structures.html#PLPGSQL-UPSERT-EXAMPLE
+
+1;
+
+
+
+=pod
+
+=head1 NAME
+
+CHI::Driver::DBI - Use DBI for cache storage
+
+=head1 VERSION
+
+version 1.24
+
+=head1 SYNOPSIS
+
+    use CHI;
+    
+    # Supply a DBI handle
+    #
+    my $cache = CHI->new( driver => 'DBI', dbh => DBI->connect(...) );
+    
+    # or a DBIx::Connector
+    #
+    my $cache = CHI->new( driver => 'DBI', dbh => DBIx::Connector->new(...) );
+    
+    # or code that generates a DBI handle
+    #
+    my $cache = CHI->new( driver => 'DBI', dbh => sub { ...; return $dbh } );
+
+=head1 DESCRIPTION
+
+This driver uses a database table to store the cache.  The newest versions of
+MySQL and SQLite work are known to work.  Other RDBMSes should work.
+
+Why cache things in a database?  Isn't the database what people are trying to
+avoid with caches?  This is often true, but a simple primary key lookup is
+extremely fast in many databases and this provides a shared cache that can be
+used when less reliable storage like memcached is not appropriate.  Also, the
+speed of simple lookups on MySQL when accessed over a local socket is very hard
+to beat.  DBI is fast.
+
+=head1 SCHEMA
+
+Each namespace requires a table like this:
+
+    CREATE TABLE chi_<namespace> (
+       `key` VARCHAR(...),
+       `value` TEXT,
+       PRIMARY KEY (`key`)
+    )
+
+The size of the key column depends on how large you want keys to be and may be
+limited by the maximum size of an indexed column in your database.
+
+The driver will attempt to create the table for you if you pass
+
+=head1 CONSTRUCTOR PARAMETERS
+
+=over
+
+=item create_table
+
+Boolean. If true, attempt to create the database table if it does not already
+exist. Defaults to false.
+
+=item namespace
+
+The namespace you pass in will be appended to the C<table_prefix> to form the
+table name.  That means that if you don't specify a namespace or table_prefix
+the cache will be stored in a table called C<chi_Default>.
+
+=item table_prefix
+
+This is the prefix that is used when building a table name.  If you want to
+just use the namespace as a literal table name, set this to undef.  Defaults to
+C<chi_>.
+
+=item dbh
+
+The main, or rw, DBI handle used to communicate with the db. If a dbh_ro handle
+is defined then this handle will only be used for writing.
+
+You may pass this handle, and dbh_ro below, in one of three forms:
+
+=over
+
+=item *
+
+a regular DBI handle
+
+=item *
+
+a L<DBIx::Connector|DBIx::Connector> object
+
+=item *
+
+a code reference that will be called each time and is expected to return a DBI
+handle, e.g.
+
+    sub { My::Rose::DB->new->dbh }
 
 =back
 
-=cut
+The last two options are valuable if your CHI object is going to live for
+enough time that a single DBI handle might time out, etc.
 
-sub get_namespaces { croak 'not supported' }
+=item dbh_ro
 
-=head1 Authors
+The optional DBI handle used for read-only operations.  This is to support
+master/slave RDBMS setups.
+
+=back
+
+=head1 AUTHORS
 
 Original version by Justin DeVuyst and Perrin Harkins. Currently maintained by
 Jonathan Swartz.
@@ -275,6 +295,19 @@ Copyright (C) Justin DeVuyst
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
+=head1 AUTHOR
+
+Jonathan Swartz <swartz@pobox.com>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2011 by Jonathan Swartz.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
 
-1;
+
+__END__
+
